@@ -51,6 +51,7 @@ fi
 if [ $1 == "test" ]; then
   echo "Test done"
 else
-  sh ./deliver-LO.sh ${pp3dv}
+  #sh ./deliver-LO.sh ${pp3dv}
+  echo "Niet"
 fi
 
