@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-__version__ = '1.6'
+__version__ = '1.7'
 __author__  = 'Olivier DUFAILLY'
 __license__ = 'BSD'
 
@@ -136,4 +136,45 @@ class LogStatus:
       self.status(ret=ret, msg=msg, *args)
       raise UsageError(msg, self)
     
+# This code was written by Krzysztof Kowalczyk (http://blog.kowalczyk.info)
+# and is placed in public domain.
+# Convert a Mozilla-style version string into a floating-point number
+#   1.2.3.4, 1.2a5, 2.3.4b1pre, 3.0rc2, etc
+def version2float(v):
+
+  def v2fhelper(v, suff, version, weight):
+    parts = v.split(suff)
+    if 2 != len(parts):
+        return v
+    version[4] = weight
+    version[5] = parts[1]
+    return parts[0]
+  
+  version = [
+      0, 0, 0, 0, # 4-part numerical revision
+      4, # Alpha, beta, RC or (default) final
+      0, # Alpha, beta, or RC version revision
+      1  # Pre or (default) final
+  ]
+  parts = v.split("pre")
+  if 2 == len(parts):
+      version[6] = 0
+      v = parts[0]
+
+  v = v2fhelper(v, "a",  version, 1)
+  v = v2fhelper(v, "b",  version, 2)
+  v = v2fhelper(v, "rc", version, 3)
+
+  parts = v.split(".")[:4]
+  for (p, i) in zip(parts, range(len(parts))):
+      version[i] = p
+  ver = float(version[0])
+  ver += float(version[1]) / 100.
+  ver += float(version[2]) / 10000.
+  ver += float(version[3]) / 1000000.
+  ver += float(version[4]) / 100000000.
+  ver += float(version[5]) / 10000000000.
+  ver += float(version[6]) / 1000000000000.
+  return ver
+
   
