@@ -2,7 +2,7 @@
 import logging
 import os, gzip
 
-from langutil import C_OK, C_FAIL, C_ERROR
+from langutil import C_OK, C_FAIL, C_ERROR, version2float
 from pypos3d.wftk.PoserFileParser import PoserFileParser, PZ3_FNAT
 
 from pypos3d.pftk.PoserBasic import getRealPath, index, PoserConst, PoserToken , isCompressed, Lang, getPoserFileType, create
@@ -11,9 +11,9 @@ from pypos3d.pftk.GeomCustom import GeomCustom
 from pypos3d.pftk.PoserMeshed import PoserProp, Camera, Light
 from pypos3d.pftk.Figure import Figure
 
-# 
+#
 #  Path Match between a referenced path of Poser and a real path.
-#  
+#
 class PathMatch(object):
 
     def __init__(self, o, r):
@@ -48,30 +48,30 @@ class GTLink():
     self.parents = {}
     self.desc = {}
     #print('Adding GTLink({:s} as {:s})'.format(qualifiedName, 'direct' if gt else 'fwd'))
-    
+
   def getParents(self):
     locset = set(self.parents.values())
     for p in self.parents.values():
       subset = set(p.getParents())
       locset |= subset
-      
+
     return list(locset)
-      
+
   def getDesc(self):
     locset = set(self.desc.values())
     for p in self.desc.values():
       subset = set(p.getDesc())
       locset |= subset
-      
+
     return list(locset)
 
-# 
+#
 class PoserFile(object):
   ''' Poser File internal representation.
   The aim of this class is to represent all kind of Poser files.
   But it works fine with : Characters (.cr2), Scenes (.pz3) and Props (.pp2)
   '''
-  
+
   # Set of channel types that can be computed
   __COMPUTABLETRANSF = { PoserToken.E_targetGeom,  PoserToken.E_valueParm, PoserToken.E_deformerPropChan, PoserToken.E_geomChan, \
                          PoserToken.E_rotateX, PoserToken.E_rotateY, PoserToken.E_rotateZ,
@@ -80,11 +80,11 @@ class PoserFile(object):
                          PoserToken.E_scale, PoserToken.E_scaleX, PoserToken.E_scaleY, PoserToken.E_scaleZ,
                          PoserToken.E_visibility
                          }
-  
+
 
   def __init__(self, fn, createLinks=False):
     ''' Create a PoserFile from a file.
-    
+
     Parameters
     ----------
     fn : str
@@ -92,52 +92,52 @@ class PoserFile(object):
     createLinks : Boolean, optional, default False
       Create the internal mapping between channels and operators
       Check referenced parts in each figure (parent, weld, figure Root)
-      
+
     Raises
     ------
     OSError : File Not found
-    
+
     '''
     super(PoserFile, self).__init__()
 
-    #  File name 
+    #  File name
     self._name = ''
-  
-    #  PoserFile type 
+
+    #  PoserFile type
     self._fileType = PoserConst.FFT_UNKNOWN
-  
-    #  Poser File Version   
+
+    #  Poser File Version
     self._version = None
-    self._versionStr = None
-  
-    #  Poser File PZ3 movieinfo   
+    self._floatVersion = 9.0 # Default version is set to Poser 9
+
+    #  Poser File PZ3 movieinfo
     self._movieinfo = None
-  
-    #  Poser File PZ3 GROUND prop   
+
+    #  Poser File PZ3 GROUND prop
     self._ground = None
-  
-    #  Poser File PZ3 FocusDistanceControl prop   
+
+    #  Poser File PZ3 FocusDistanceControl prop
     self._focusDistanceControl = None
-  
-    #  First actor of a PZ3 file 
+
+    #  First actor of a PZ3 file
     self._universe = None
-  
-    #  List of "Free" props (Prop, hairProp, controlProp)  = Not Attached to any character 
+
+    #  List of "Free" props (Prop, hairProp, controlProp)  = Not Attached to any character
     self._lstProp = [ ]
-  
-    #  List of managed Camera 
+
+    #  List of managed Camera
     self._lstCamera = [ ]
-  
-    #  List of managed Lights 
+
+    #  List of managed Lights
     self._lstLight = [ ]
-  
-    #  List of managed Hair growth groups 
+
+    #  List of managed Hair growth groups
     self._lstGrowtGroup = [ ]
-  
-    #  List of Figure 
+
+    #  List of Figure
     self._lstFigure = [ ]
-  
-    #  Poser File PZ3 doc   
+
+    #  Poser File PZ3 doc
     self._doc = None
     self._illustrationParms = None
     self._renderDefaults = None
@@ -145,13 +145,15 @@ class PoserFile(object):
 
     # Channels link Dictionnary
     self._gtlnk = {}
-  
-    #  Temporary Attributes 
+
+    #  Temporary Attributes
     self._lastBodyIdx = 0
-  
-    #  Last read (or discovered) figure at read time 
+
+    #  Last read (or discovered) figure at read time
     self._lastFigure = None
     self._lastFRF = None
+    # Reference to an optional Binary Morph File (not supported for morph operations)
+    self._lastMBF = None
 
     self.setName(fn)
     ft = getPoserFileType(fn)
@@ -168,52 +170,55 @@ class PoserFile(object):
 #       exc_type, exc_value, exc_tb = sys.exc_info()
 #       traceback.print_exception(exc_type, exc_value, exc_tb)
 #       print('Exception in PoserFile.__init__:'+str(pfr.lineno()))
-           
+
     rin.close()
     logging.info("File[%s] : %d lines read", fn,  pfr.lineno())
-    
+
     if createLinks:
       self.__createLinks(True)
       self.checkActorLinks()
-      
-    #  if WFBasic.PYPOS3D_TRACE: print ('File({0:s} - Read Error {1:s}'.format(fn, str(e.args)))
-      
 
-  # 
-  def getName(self): 
+    #  if WFBasic.PYPOS3D_TRACE: print ('File({0:s} - Read Error {1:s}'.format(fn, str(e.args)))
+
+
+  #
+  def getName(self):
     ''' Return the name of the object, for a Poser file the name is the filename. '''
     return self._name
 
-  # 
+  #
   # * Set the name.
   # @param string
-  #    
+  #
   def setName(self, s):
     self._name = s
 
-  # 
+  #
   # Return the Poser Language Version as an PoserObject
-  #    
+  #
   def getVersion(self): return self._version
 
-  def isVersion(self, cmpVers):
-    if not self._versionStr:
-      sa = self._version.findAttribut(PoserToken.E_number)
-      self._versionStr = sa.getValue() if sa else ''
+#  def isVersion(self, cmpVers):
+#    if not self._versionStr:
+#      sa = self._version.findAttribut(PoserToken.E_number)
+#      self._versionStr = sa.getValue() if sa else ''
+#    return False if self._versionStr==None else (self._versionStr==cmpVers)
 
-    return False if self._versionStr==None else (self._versionStr==cmpVers)
+  def isBetterVersion(self, v):
+    ''' Test if current PoserFile version is better (younger) then given one. '''
+    return self._floatVersion >= (version2float(v) if isinstance(v, str) else v)
 
-  # 
+  #
   # Read a structured attribute data from the file. This class name, the name
   # and the opening bracket are supposed to be consumed by the caller.
   # Because it is the mean it uses to recognize a structured attribute.
-  #    
+  #
   def read(self, st):
     code,cn,rw = st.getLine()
-    
+
     while (code!=PoserFileParser.TT_EOF) and (code!=PoserFileParser.TT_RIGHTBRACKET):
 
-      if code== PoserFileParser.TT_WORD:
+      if code==PoserFileParser.TT_WORD:
         try:
           vc = Lang[cn]
           #  Known word
@@ -227,7 +232,7 @@ class PoserFile(object):
               #  We're not reading a CR2 or PP2 or PZ3 file, but
               #  probably a Pose file ... nothing to do
               logging.warning("Line[%d]: Figure block found without figure", st.lineno())
-              
+
           elif vc.isStructured:
             nom = rw # "" if st.ttype == PoserFileParser.TT_EOL else st.sval
             sta = create(vc, nom)
@@ -239,70 +244,81 @@ class PoserFile(object):
             code,cn,rw = st.getLine()
             if code==PoserFileParser.TT_LEFTBACKET:
               sta.read(st)
+
+              # Poser Version Management : Check if we've met version {}
+              # and record it in the reader to ease decoding
+              if sta.getPoserType()==PoserToken.E_version:
+                vstr = sta.getAttribut(PoserToken.E_number).getValue()
+                vstr = vstr if vstr else PoserConst.POSER_V9
+                self._floatVersion = version2float(vstr)
+                st.setFileVersion(self._floatVersion)
+
             else:
               logging.error("Line[%d]:  '{' is missing for %s", st.lineno(), cn)
+          elif vc.isDirect:
+            logging.error("Line[%d]: setDirect() illegal at PoserFile level %s", st.lineno(), cn)
           else:
-            if vc.isDirect:
-              #st.getToNextLine()
-              logging.error("Line[%d]: setDirect() illegal at PoserFile level %s", st.lineno(), cn)
-            else:
-              # Read before add
-              sa = create(vc, cn)
-              sa.read(st, rw)
-              self.addAttribut(sa)
+            # Read before add
+            sa = create(vc, cn)
+            sa.read(st, rw)
+            self.addAttribut(sa)
 
-            #nextLine = False
         except KeyError: # if vc == None: --> Unknow word
           logging.info("Line[%d] - Unknown word:%s", st.lineno() , cn)
 
       #  Get next token
       code,cn,rw = st.getLine()
       # Wend
-  # 
-  # Add an object at highest level of the Poser File.
-  #    
+  #
   def addAttribut(self, po):
+    ''' Add an object at highest level of the Poser File. '''
     pot = po.getPoserType()
 
     if pot==PoserToken.E_version:
       self._version = po
-      return self._version
+      return po
 
     elif pot==PoserToken.E_movieInfo:
       self._movieinfo = po
-      return self._movieinfo
+      return po
 
     elif pot==PoserToken.E_keyLayer:
-      logging.error("Ignored " + po.getName() + "[" + po.getPoserType() + "]")
+      logging.error("Ignored {po.getName()} [{po.getPoserType()}]")
       return po
 
     elif pot==PoserToken.E_figureResFile:
       #  Beginning of a figure
       self._lastFRF = po
       fig = self.findFigure(self._lastFRF)
-      if fig == None:
+      if fig is None:
         fig = Figure(self, frf=self._lastFRF)
         self._lstFigure.append(fig)
         self._lastFRF = None
-      #  else  keep the lastFRF, because could be used for another figure with same geometry      
+      #  else  keep the lastFRF, because could be used for another figure with same geometry
       self._lastFigure = fig
       return None
 
+    elif pot==PoserToken.E_morphBinaryFile: # Signal that Binary morphs are not supported
+      # Same Behavior as figureResFile and stored within it
+      self._lastMBF = po
+      logging.warning(f"BinaryMorph not supported for morph operations:{po.getName()}")
+      return None
+
     elif pot==PoserToken.E_actor:
       pa = po
       pa.setPoserFile(self)
       if pa.getName() == PoserConst.C_UNIVERSE:
-        if self._universe == None:
+        if self._universe is None:
           self._universe = pa
         return self._universe
 
       idx = pa.getIndex()
 
-      if self._lastFigure == None:
+      if self._lastFigure is None:
         self._lastFigure = Figure(self, idx)
         self._lstFigure.append(self._lastFigure)
         if (self._fileType==PoserConst.PFT_CR2) or (self._fileType==PoserConst.PFT_PZ3):
-          logging.warning("Actor " + pa.getName() + " without figure in " + self.getName())
+          logging.warning(f"Actor {pa.getName()} without figure in {self.getName()}")
 
       else:
         if self._lastFigure.getBodyIndex()==PoserConst.C_BAD_INDEX:
@@ -312,12 +328,12 @@ class PoserFile(object):
         else:
           #  Check index coherence
           if self._lastFigure.getBodyIndex() != idx:
-            #  Means that several figures have the same target OBJ file 
+            #  Means that several figures have the same target OBJ file
             nf = self.getFigure(idx)
             if nf:
               self._lastFigure = nf
             else:
-              #  log.warning("Figure index [" + idx + "] does not exists for actor " + pa.getName());              
+              #  log.warning("Figure index [" + idx + "] does not exists for actor " + pa.getName());
               self._lastFigure = Figure(self, frf=self._lastFRF)
               self._lstFigure.append(self._lastFigure)
               self._lastFigure.setBodyIndex(idx)
@@ -332,7 +348,7 @@ class PoserFile(object):
       pp = po
       pp.setPoserFile(self)
       if pp.getName() == "GROUND":
-        if self._ground == None:
+        if self._ground is None:
           self._ground = pp
         return self._ground
 
@@ -340,7 +356,7 @@ class PoserFile(object):
         if pp.getName() == epp.getName():
           return epp
 
-      if self._lastFigure == None:
+      if self._lastFigure is None:
         #  We are in the declarative part of "free" props
         self._lstProp.append(pp)
       else:
@@ -373,8 +389,9 @@ class PoserFile(object):
          pot==PoserToken.E_magnetDeformerProp or \
          pot==PoserToken.E_sphereZoneProp or \
          pot==PoserToken.E_curveProp or \
-         pot==PoserToken.E_waveDeformerProp or\
-         pot==PoserToken.E_coneForceFieldProp:
+         pot==PoserToken.E_waveDeformerProp or \
+         pot==PoserToken.E_coneForceFieldProp or \
+         pot==PoserToken.E_controlHandleProp :
       pp = po
       pp.setPoserFile(self)
       if pp.getName() == "FocusDistanceControl":
@@ -417,51 +434,51 @@ class PoserFile(object):
 
     return None
 
-  # 
+  #
   # Return the figure associated to the BODY index
   # * @param idx
   # *          The BODY index
   # * @return
-  #    
+  #
   def getFigure(self, idx):
-    for fig in self._lstFigure:
-      if fig.getBodyIndex() == idx:
-        return fig
-    return None
+    # for fig in self._lstFigure:
+    #   if fig.getBodyIndex() == idx:
+    #     return fig
+    return next ( (fig for fig in self._lstFigure if fig.getBodyIndex() == idx), None)
 
-  # 
+  #
   # Returns the list of figures contained in the Poser File
   # @return the list of Figure objects (not a copy)
-  #    
+  #
   def getLstFigure(self): return self._lstFigure
 
-  # 
+  #
   # Returns the list of Cameras contained in the Poser File
   # @return the list of Camera objects (not a copy)
-  #    
+  #
   def getLstCamera(self): return self._lstCamera
 
-  # 
+  #
   # Returns the list of Lights contained in the Poser File
   # @return the list of Light objects (not a copy)
-  #    
+  #
   def getLstLight(self): return self._lstLight
 
-  # 
+  #
   # Returns the list of Props contained in the Poser File
   # @return the list of PoserProp objects (not a copy)
-  #    
+  #
   def getLstProp(self): return self._lstProp
 
-  # 
+  #
   # Return the list of all objects names that can be referenced.
   # @return
-  #    
+  #
   def getLstFigProp(self):
     return [ pp for pp in self.getLstProp() ] +\
-           [ n for f in self.getLstFigure() for n in f.getActors() ] 
-   
-   
+           [ n for f in self.getLstFigure() for n in f.getActors() ]
+
+
   def findFigure(self, frf):
     for fig in self._lstFigure:
       if frf.getValue() == fig.getFigResFile().getValue():
@@ -487,30 +504,30 @@ class PoserFile(object):
 
     return None
 
-  # 
-  # Return the list of PoserMeshedObject (actor, prop, hairprop) 
+  #
+  # Return the list of PoserMeshedObject (actor, prop, hairprop)
   # Return a list of PoserMeshedObject (actor, prop, hairprop) that match the
   #  given nameif any
   # @return a list of PoserMeshedObject
-  #    
+  #
   def findAllMeshedObject(self, actorPropName=None):
     if actorPropName:
       return [ pp for pp in self._lstProp + self._lstLight+self._lstCamera if actorPropName == pp.getName() ] + \
             [ part for fig in self._lstFigure for part in fig.getActors() if actorPropName == part.getName() ]
     else:
       return self._lstProp + self._lstLight + self._lstCamera + \
-        [ a for fig in self._lstFigure for a in fig.getActors() ] 
+        [ a for fig in self._lstFigure for a in fig.getActors() ]
 
-  # 
+  #
   def writeFile(self, fn):
     ''' Write PoserFile in text format (without compression)
     # RENAMED Java write --> Python writeFile
-     
+
     Parameters
     ----------
     fn : str
       Full path name
-      
+
     Returns
     -------
     int
@@ -518,19 +535,19 @@ class PoserFile(object):
     '''
     ret = C_OK
     try:
-      logging.info("File[" + fn + "] : writing")
+      logging.info(f"File[{fn}] : writing")
       prn = open(fn, 'w')
       self.write(prn)
       prn.close()
-      logging.info("File[" + fn + "] : " + str(os.path.getsize(fn)) + " bytes")
+      logging.info(f"File[{fn}]: {os.path.getsize(fn)} bytes")
     except OSError:
       ret = C_ERROR
     return ret
 
-  # 
+  #
   def writeZ(self, fn):
     ''' Write PoserFile in Zlib format (text stream compressed)
-     
+
     Parameters
     ----------
     fn : str
@@ -543,11 +560,11 @@ class PoserFile(object):
     '''
     ret = C_OK
     try:
-      logging.info("File[" + fn + "] : writing/zipping")
-      out = gzip.open(fn, 'wt') 
+      logging.info(f"File[{fn}]: writing/zipping")
+      out = gzip.open(fn, 'wt')
       self.write(out)
       out.close()
-      logging.info("File[" + fn + "] : " + str(os.path.getsize(fn)) + " bytes")
+      logging.info(f"File[{fn}]: {os.path.getsize(fn)} bytes")
     except OSError:
       ret = C_ERROR
     return ret
@@ -576,9 +593,9 @@ class PoserFile(object):
 
 
 
-  # 
+  #
   # @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
-  #    
+  #
   def write(self, fw):
     isPZ3 = (self._fileType & PoserConst.PFT_MASK) == PoserConst.PFT_PZ3
 
@@ -586,6 +603,10 @@ class PoserFile(object):
     if self._version:
       self._version.write(fw, "")
 
+    # Write the Binary File name
+    if self._lastMBF:
+      self._lastMBF.write(fw, "")
+
     if isPZ3 and self._movieinfo:
       self._movieinfo.write(fw, "")
 
@@ -594,7 +615,7 @@ class PoserFile(object):
     #  Write Definition of Cameras
     #  Write Definition of Figures
     allParts = [ x for x in [ self._ground, self._universe, self._focusDistanceControl ] if isPZ3  and x ]
-    allParts += self._lstProp + self._lstLight + self._lstCamera + self._lstFigure 
+    allParts += self._lstProp + self._lstLight + self._lstCamera + self._lstFigure
     for part in allParts:
       part.writeDef(fw, "")
 
@@ -625,22 +646,22 @@ class PoserFile(object):
   def getChannel(self, groupName, channelName):
     po = self.findMeshedObject(groupName)
     return po.getGenericTransform(channelName) if po else None
-  
+
   # Return the ancestors channel list of the channel given by channelName in
   # groupName actor.
-  # 
+  #
   # @param figureName
   #          Unused because of welknown CrossWalk problems
   # @param groupName
   # @param channelName
   # @return The list of GenericTransform
   def getChannelAncestor(self, figureName, groupName, channelName):
-    lstact = self.findAllMeshedObject(groupName)   
+    lstact = self.findAllMeshedObject(groupName)
     if lstact and (len(lstact)==1):
       po = lstact[0]
-      
+
       channel = po.getChannel(channelName) # next(attr for attr in po.getChannels() if attr.getName()==channelName)
-      
+
       if not self._gtlnk:
         self.__createLinks(True)
 
@@ -656,16 +677,16 @@ class PoserFile(object):
     logging.warning(figureName + "." + groupName + " Not Found")
     return []
 
-  # 
+  #
   # Return the k factor of the first frame of the channel given by groupName of
   # the group given by groupName.
-  # 
+  #
   # * @param figureName
   # *          Unused because of well known CrossWalk problems
   # * @param groupName
   # * @param channelName
   # * @return The float factor.
-  #    
+  #
   def getFactor(self, figureName, groupName, channelName):
     po = self.findActor(groupName)
     if po:
@@ -681,9 +702,9 @@ class PoserFile(object):
               # Get the real factor of the master
               f = self.getFactor(vod.getFigureName(), vod.getGroupName(), vod.getChannelName())
               finalFactor = vod.calc(finalFactor, f)
-                    
+
           return finalFactor
-        
+
       logging.warning("No Channel [%s] for [%s.%s]", channelName, figureName, groupName)
 
     logging.warning(figureName + "." + groupName + " Not Found")
@@ -699,7 +720,7 @@ class PoserFile(object):
 #           # Found : Already in the list
 #           found = True
 #           break
-# 
+#
 #       if not found:
 #         newgt = self.getChannel(op.getGroupName(), op.getChannelName())
 #         if newgt:
@@ -715,7 +736,7 @@ class PoserFile(object):
 
   def getDescendant(self, bodyIdx, firstName=None):
     f = self.getFigure(bodyIdx)
-    return f.getDescendant(firstName) if f else None 
+    return f.getDescendant(firstName) if f else None
 
   def getWelded(self, bodyIdx, firstName):
     fig = self.getFigure(bodyIdx)
@@ -741,7 +762,7 @@ class PoserFile(object):
     if (ficpath==None) or (ficpath=="") or (ficpath=='""') or (ficpath=="NO_MAP"):
       return
 
-    try: # pm = 
+    try: # pm =
       hshres[ficpath]
     except KeyError:
       # Clean the ficpath
@@ -799,8 +820,26 @@ class PoserFile(object):
     #logging.info("List size:" + str(len(lstres)))
     return lstres
 
+
+  def __removePMODeltas(self, pp, setTargetMorph):
+    for attr in pp.getChannels():
+      if attr.getPoserType()==PoserToken.E_targetGeom and GenericTransform.concerned(attr.getName(), setTargetMorph):
+        finalFactor = 0.0
+        for vod in attr.getVOD():
+          # Get the real factor of the master
+          f = self.getFactor(vod.getFigureName(), vod.getGroupName(), vod.getChannelName())
+          # Take into account the operator kind
+          finalFactor = vod.calc(finalFactor, f)
+
+        finalFactor += attr.getKeysFactor(0)
+
+        # Remove the deltas but not the targetGeom
+        logging.info("Cleaning[%s] in %s f=%g", pp.getName(), pp.getName(), finalFactor)
+        attr.removeDeltas()
+
+
+
   # TODO: Ugly interface to change
-  # TODO: Duplicate code to clean
   def cleanNonNullDelta(self, bodyIdx:int=-1, pp=None, setTargetMorph=None):
     ''' Remove deltas from targetGeom in a Figure or a PoserProp or a PoserActor.
     Parameters
@@ -819,54 +858,24 @@ class PoserFile(object):
     ret = C_OK
     if bodyIdx>=0:
       logging.info("------------------------ Cleaning Deltas ------------------------")
-      # Remove unused attributes of the PoserObject
-      # Get all part of the character
+      # Get all part of the Figure
       lstActor = self.getDescendant(bodyIdx)
       if lstActor:
-        # Channels where not found when Figure index was higher than 9 (2 digits)
-        #idxRelPos = 3 if (bodyIdx > 9) else 2
-
         for po in lstActor:
-          for attr in po.getChannels():
-            if attr.getPoserType()==PoserToken.E_targetGeom and GenericTransform.concerned(attr.getName(), setTargetMorph):
-              finalFactor = 0.0
-              for vod in attr.getVOD():
-                # Get the real factor of the master
-                f = self.getFactor(vod.getFigureName(), vod.getGroupName(), vod.getChannelName())
-                # Take into account the operator kind
-                finalFactor = vod.calc(finalFactor, f)
-
-              # Deltas dltAttr = attr.getDeltas();
-              finalFactor += attr.getKeysFactor(0)
-              logging.info("Cleaning[%s] in %s f=%g", attr.getName(), po.getName(), finalFactor)
-              attr.removeDeltas()
+          self.__removePMODeltas(po, setTargetMorph)
       else:
         ret = C_ERROR
 
     else: # pp should be a PoserProp
       if not pp: return C_ERROR
-      
-      for attr in pp.getChannels():
-        if attr.getPoserType()==PoserToken.E_targetGeom and GenericTransform.concerned(attr.getName(), setTargetMorph):
-          finalFactor = 0.0
-          for vod in attr.getVOD():
-            # Get the real factor of the master
-            f = self.getFactor(vod.getFigureName(), vod.getGroupName(), vod.getChannelName())
-            # Take into account the operator kind
-            finalFactor = vod.calc(finalFactor, f)
-
-          finalFactor += attr.getKeysFactor(0)
-
-          # Remove the deltas but not the targetGeom
-          logging.info("Cleaning[%s] in %s f=%g", pp.getName(), pp.getName(), finalFactor)
-          attr.removeDeltas()
+      self.__removePMODeltas(pp, setTargetMorph)
 
     return ret
 
 
   # Extract the geometries of a Figure, with or without taking into account the
   # applied morph.
-  # 
+  #
   # @param poserRootDir
   #          Path to Poser install
   # @param bodyIdx
@@ -891,11 +900,11 @@ class PoserFile(object):
           eg = body.extractSortGeom(pa.getGeomGroupName(), pa.getGeomType())
 
         elif pa.getGeomType()==PoserConst.GT_LOCAL_OBJFILE:
-          # objFileGeom 0 0 :Runtime:Geometries:Marforno:Elaya:Lamp.obj          
+          # objFileGeom 0 0 :Runtime:Geometries:Marforno:Elaya:Lamp.obj
           eg = GeomCustom(getRealPath(poserRootDir, pa.getGeomFileName()))
 
         #elif pa.getGeomType()==PoserConst.GT_NONE:
-        #elif pa.getGeomType()==PoserConst.GT_INTERNAL: 
+        #elif pa.getGeomType()==PoserConst.GT_INTERNAL:
         else:
           eg = None
 
@@ -910,20 +919,19 @@ class PoserFile(object):
   # @param channel
   def deleteChannelImpact(self, channel):
     impset = { channel, }
-    
+
     if self._gtlnk:
       qn = channel.getPoserMeshedObject().getName() + '.' + channel.getName()
       gtl = self._gtlnk[qn]
       impset.update( descgtl.gt for descgtl in gtl.desc.values() )
-    else:  
+    else:
       for f in self.getLstFigure():
         for pa in f.getActors():
           pa.getChannelDescendant(channel, impset)
-#         for pp in f.getProps():
-#           pp.getChannelDescendant(channel, impset)
+
       for pp in self.getLstProp():
         pp.getChannelDescendant(channel, impset)
-        
+
     return impset
 
   # Calculate the set of channels that are referenced at any level by the given channel. (recursive search)
@@ -953,7 +961,7 @@ class PoserFile(object):
     lstgtl = gtl.getDesc()
 
     return [ descgtl.gt for descgtl in lstgtl ]
-    
+
 
   def printChannelLinks(self, channel):
     ''' Return a tuple of printable data for the channel (Qualified Name, nbDescDelta, DescChannelsNames, ParentChannelsNames) '''
@@ -968,27 +976,27 @@ class PoserFile(object):
         return (channel, -1, '', '')
     else:
       gt = channel
-      
+
     # Compute channel dependencies
     nbdeltas, labparent, labdesc = 0, '',''
     try:
       lparent = self.calcChannelParent(gt)
       for gtdesc in [ g for g in lparent if g ]:
         if labparent:
-          labparent += ', '          
+          labparent += ', '
         labparent += gtdesc.getQualifiedName()
-      
+
       ldesc = self.calcChannelDesc(gt)
       for gtdesc in [ g for g in ldesc if g ]:
         nbdeltas += len(gtdesc._dlt.deltaSet) if gtdesc._dlt and gtdesc.getPoserType()==PoserToken.E_targetGeom else 0
         if labdesc:
           labdesc += ', '
         labdesc += gtdesc.getQualifiedName()
-      
+
     except KeyError:
       # Channel does not exist
       return (channel, -1, '', '')
-    
+
     return (channel.getQualifiedName(), nbdeltas, labdesc, labparent)
 
   def __createLinks(self, printErrors=False):
@@ -1016,12 +1024,12 @@ class PoserFile(object):
         for vop in gt.getVOD():
           # Check valueOp syntax, the figure index seems to be optional
           gn = vop.getGroupName()
-          
+
           # Search for the channel as given by the valueOp
           refqn = gn + '.' +  vop.getChannelName()
           try:
             refgtl = self._gtlnk[refqn]
-            
+
           except KeyError:
             p = gn.rfind(':')
             if p<0:
@@ -1049,8 +1057,8 @@ class PoserFile(object):
     if printErrors and nbdeclfwd>0:
       for k,gtl in self._gtlnk.items():
         if not gtl.gt:
-          print('Not Declared Channel:{:s}'.format(k))
-  
+          print(f'Not Declared Channel:{k}')
+
   def getGTLinks(self):
     if not self._gtlnk:
       self.__createLinks(True)
@@ -1058,9 +1066,9 @@ class PoserFile(object):
 
   def delGTLink(self, gtOrQn):
     if not self._gtlnk: return
-    
+
     qn = gtOrQn.getQualifiedName() if isinstance(gtOrQn, GenericTransform) else gtOrQn
-      
+
     del self._gtlnk[qn]
     for gtl in self._gtlnk.values():
       gtl.desc.pop(qn, None)
@@ -1074,47 +1082,38 @@ class PoserFile(object):
   def deleteChannel(self, channel, lstImp=None):
     if lstImp == None:
       lstImp = self.deleteChannelImpact(channel)
-    
+
     lstImp.remove(channel)
-    
+
     # Remove calculated links
     self.delGTLink(channel)
-    
+
     pmo = channel.getPoserMeshedObject()
     pmo.deleteChannel(channel)
     for ci in lstImp:
       ci.deleteChannelRef(channel)
+
+  def __removeFromDoc(self, poserType, name):
+    for po in self._doc.getLstAttr():
+      if (po.getPoserType() == poserType) and (po.getValue()==name):
+        self._doc.getLstAttr().remove(po)
     
 
-
-
-
-#   def deleteFigureImpact(self, fig):
-#     logging.error("Not implemented yet")
-#     return None
-
   def delete(self, obj):
-    '''  Delete the given PoserProp or Figure from the list of general props '''
+    '''  Delete the given PoserProp or Figure or Camera or Light from the list of general props. '''
     res = C_OK
-    if isinstance(obj, PoserProp):
-      try:
+    try:
+      if isinstance(obj, PoserProp):
         self._lstProp.remove(obj)
         propName = obj.getName()
 
         # Clean Figure Attributs : addChild and weld
-        for po in self._doc.getLstAttr():
-          if (po.getPoserType() == PoserToken.E_addActor) and (po.getValue()==propName):
-            self._doc.getLstAttr().remove(po)
-
-      except:
-        res = C_FAIL
-    elif isinstance(obj, Camera):
-      try:
+        self.__removeFromDoc(PoserToken.E_addActor, propName)
+  
+      elif isinstance(obj, Camera):
         self._lstCamera.remove(obj)
         propName = obj.getName()
-        for po in self._doc.getLstAttr():
-          if (po.getPoserType() == PoserToken.E_addCamera) and (po.getValue()==propName):
-            self._doc.getLstAttr().remove(po)
+        self.__removeFromDoc(PoserToken.E_addCamera, propName)
 
         # Relink Lights, if the parent of the deleted camera was not "UNIVERSE"
         if not obj.getParent() == PoserConst.C_UNIVERSE:
@@ -1123,31 +1122,23 @@ class PoserFile(object):
             if sa and (sa.getValue() == propName):
               # What value to set?? instead : Name of the first Camera of the file
               sa.setValue(self._lstCamera[0].getName())
-      except:
-        res = C_FAIL
-
-    elif isinstance(obj, Light):
-      try:
+  
+      elif isinstance(obj, Light):
         self._lstLight.remove(obj)
         propName = obj.getName()
-        for po in self._doc.getLstAttr():
-          if (po.getPoserType() == PoserToken.E_addLight) and (po.getValue()==propName):
-            self._doc.getLstAttr().remove(po)
+        self.__removeFromDoc(PoserToken.E_addLight, propName)
 
-        # Check the 'parent' of each Camera and relikn if needed
-        firstLight = self._lstLight[0]
-        for cam in self._lstCamera:
-          if cam.getParent() == propName:
-            if firstLight == None:
-              logging.warning("Camera[" + cam.getName() + "] can not be relinked (no more lights)")
-            else:
-              cam.setParent(firstLight.getName())
-              logging.info("Camera[" + cam.getName() + "] relinked to " + firstLight.getName())
-      except:
-        res = C_FAIL
+        # Check the 'parent' of each Camera and relink it if needed
+        firstLight = self._lstLight[0] if self._lstLight else None
 
-    elif isinstance(obj, Figure):
-      try:
+        if firstLight is None:
+          logging.warning("Cameras can not be relinked (no more lights)")
+        else:
+          for cam in [c for c in self._lstCamera if c.getParent() == propName]:
+            cam.setParent(firstLight.getName())
+            logging.info("Camera[" + cam.getName() + "] relinked to " + firstLight.getName())
+  
+      elif isinstance(obj, Figure):
         self._lstFigure.remove(obj)
         fi = obj.getBodyIndex()
         lstprp = self.findAllMeshedObject()
@@ -1161,16 +1152,21 @@ class PoserFile(object):
           # Delete any references in channels of actors of other figures and Props
           for pgt in pmo.getChannels():
             pgt.deleteFigureRef(obj)
-
-      except:
+            
+      else: # Unknown object class
+        logging.warning(f'Unknow Object Class:{obj.getName() if obj else "None"}')
         res = C_FAIL
-        
-    # Remove calculated links (anyway, to be consistent)
-    self._gtlnk.clear()
+  
+      # Remove calculated links (anyway, to be consistent)
+      self._gtlnk.clear()
+    
+    except ValueError:
+      logging.warning(f'Object not found:{obj.getName()}')
+      res = C_FAIL
 
     return res
 
-  # Change the name of referenced part (actor, prop, hairProp, controlProp)  
+  # Change the name of referenced part (actor, prop, hairProp, controlProp)
   # @param oldPartName
   # @param newPartName
   def changeReference(self, oldPartName, newPartName):
@@ -1185,19 +1181,19 @@ class PoserFile(object):
 
     logging.info("Linking[%s]:Actors", self.getName())
     nberrors=0
-    
+
     for fig in self._lstFigure:
       nberrors += fig.checkActorLinks()
 
     for act in self._lstProp:
       parentName = act.getParent()
-      
+
       if parentName==PoserConst.C_UNIVERSE:
         if self._universe:
           continue
         else:
           nberrors+=1
-          logging.warn('Missing Declared Parent Actor: %s for %s', parentName, act.getName())        
+          logging.warn('Missing Declared Parent Actor: %s for %s', parentName, act.getName())
       else:
         p = self.findActor(parentName)
         if not p:
@@ -1206,27 +1202,27 @@ class PoserFile(object):
 
     if nberrors:
       logging.info("Linking[%s]:Actors with %d errors", self.getName(), nberrors)
-      
+
     return C_OK if nberrors==0 else C_FAIL
-      
-      
+
+
   def createFigure(self, name=None):
     ''' Create a new Figure object, initialize the correct index. '''
     bodyIdx = 1 + (max( f.getBodyIndex() for f in self.getLstFigure() ) if self.getLstFigure() else 0)
-     
+
     fig = Figure(self, idx=bodyIdx)
-    
+
     if name:
       fig.setPrintName(name)
-    
+
     self._lstFigure.append(fig)
-    
+
     # At this stage the Figure is not consistent (no geom, no actor, ...)
-    
+
     logging.info("Figure Created: [%s]:%d", name if name else '', bodyIdx)
     return fig
-      
-      
-      
+
+
+
 
 
