@@ -15,7 +15,7 @@ from pypos3d.pftk.StructuredAttribut import StructuredAttribut, AlternateGeom, D
 
 #
 # Options of the report algorithm
-# 
+#
 ReportOption = namedtuple('ReportOption', ['translation', \
   'maxDist', # Value changed to fit a good sphere for DAZ characters \
   'enhance',\
@@ -26,63 +26,63 @@ ReportOption = namedtuple('ReportOption', ['translation', \
 
 ReportOption.__new__.__defaults__ = ( Vector3d(), 0.025, PoserConst.C_NO_ENHANCEMENT, PoserConst.C_NO_BOUNDING, True, 0.6, 0.001)
 
-#  
+#
 class PoserMeshedObject(StructuredAttribut):
   ''' A PoserMeshedObject represents either a Prop, an Actor, a Camera, a Light or hairs in a Poser File. '''
 
   def __init__(self, Name=None):
     super(PoserMeshedObject, self).__init__(Name)
 
-    #  Figure Object (may be null) 
+    #  Figure Object (may be null)
     self._fig = None
     self._pmName = ''
     self._hidden = False
     self._visible = True #  On
     self._bend = True #  1
-  
-    #  Poser V9 attribut 
+
+    #  Poser V9 attribut
     self._animatableOrigin = False
-    self._dynamicsLock = True #        0
-  
-    #  Poser V9 attribut 
-    self._collisionDetection = False
-    self._addToMenu = True #     1
-    self._castsShadow = True #       1
-    self._includeInDepthCue = True #         1
-    self._useZBuffer = True #        1
+    self._dynamicsLock = True
+
+    #  Poser V9 attribut
+    self._collisionDetection  = None # Seems to be optional (in Poser11) = False
+    self._addToMenu = True
+    self._castsShadow = True
+    self._includeInDepthCue = True
+    self._useZBuffer = True
     self._parent = None
     self._conformingTarget = None
     self._creaseAngle = 80.0
-  
-    #  Storage class of the mesh 
+
+    #  Storage class of the mesh
     self._geomType = PoserConst.GT_NONE
-  
-    #  Geom Groupname when geomType is GLOBAL_OBJFILE 
+
+    #  Geom Groupname when geomType is GLOBAL_OBJFILE
     self._geomGroupName = None
-  
-    #  Geom filename when geomType is LOCAL_OBJFILE, formated like Poser with ':' or ""  
+
+    #  Geom filename when geomType is LOCAL_OBJFILE, formated like Poser with ':' or ""
     self._geomFileName = None
-  
+
     #  Base Geometry
     self._geomCustom = None
     self._geom = None
-  
-    #  FigureResFile with a path (relative) 
+
+    #  FigureResFile with a path (relative)
     self._FRF = None
-  
-    #  Channels = Ordered Dictionary of Channels (only GenericTransform) - Never null 
+
+    #  Channels = Ordered Dictionary of Channels (only GenericTransform) - Never null
     #  { ChannelName in lower char : GenericTransform }
     self._channels = { }
-    #  Store the group presentation of channels 
+    #  Store the group presentation of channels
     self._groups = None
-  
-    #  endPoint position 
+
+    #  endPoint position
     self._endPoint = None
-  
-    #  origin of the mesh 
+
+    #  origin of the mesh
     self._origin = None
-  
-    #  orientation of the mesh 
+
+    #  orientation of the mesh
     self._orientation = None
     self._displayOrigin = False
     self._displayMode = None #  USEPARENT
@@ -92,7 +92,14 @@ class PoserMeshedObject(StructuredAttribut):
     self._lstWMap = [ ] # List of Weight Maps (could be implemented with a ordered dict)
 
     self._customData = None # Contain a Dictionnary of 'key' : custData()
-  
+
+    # Poser11 attributes
+    self._subdivLevels = 0
+    self._subdivRenderLevels = 0
+    self._subdivideWithFigure = None # Seems to be optional
+    self._backfaceBehavior = 1
+    self._animatableOrientation = False
+
     #  Name + Mesh definition
     self._printName = None
     self._pf = None
@@ -108,7 +115,7 @@ class PoserMeshedObject(StructuredAttribut):
 
   def getPoserFile(self): return self._pf
 
-  def getChannels(self): 
+  def getChannels(self):
     ''' Return a View on the channels '''
     return self._channels.values()
 
@@ -140,20 +147,20 @@ class PoserMeshedObject(StructuredAttribut):
     Limit search to CustomMaterial if inCustom=True
     '''
     mat = None
-    
+
     if self.getCustomMaterial():
       mat = next( (m for m in self.getLstAttr() \
                   if (m.getPoserType()==PoserToken.E_material) and (matName=='*' or matName==m.getName())), None)
-        
+
     if not mat and not inCustom and self.getFigure():
       mat = self.getFigure().getMaterial(matName)
-          
+
     return mat
 
   # Return the list of alternate geom (never null in Python)
   def getAltGeomList(self): return self._lstAltGeom
 
-  #    
+  #
   def addAltGeom(self, ag=None, altGeomFile=None, poserRootDir=None, pos=-1):
     ''' Add an alternate Geometry, if it does not already exists.
     Returns
@@ -180,7 +187,7 @@ class PoserMeshedObject(StructuredAttribut):
       except IOError:
         ret = C_ERROR
 
-   
+
     #  Add the AlternateGeom to the specified position
     if pos>=0:
       self._lstAltGeom.insert(pos, ag)
@@ -195,21 +202,21 @@ class PoserMeshedObject(StructuredAttribut):
     for i,a in enumerate(self._lstAltGeom):
       a.setNo(i)
 
-  # 
+  #
   # Move an alternate Geometrie to an upper position
-  # @return C_OK, C_FAIL if the geometrie was already existing, 
+  # @return C_OK, C_FAIL if the geometrie was already existing,
   #         PoserConst.C_ERROR in case data inconsistency
-  #    
+  #
   def moveAltGeom(self, ag):
     ret = self.removeAltGeom(ag)
     if ret == C_OK:
       ret = self.addAltGeom(ag, ag.getNo())
     return ret
 
-  # 
+  #
   # Remove an alternate Geometrie
   # @return C_OK, C_FAIL if the alt geometrie was not existing
-  #    
+  #
   def removeAltGeom(self, ag):
     res = C_OK
     try:
@@ -220,16 +227,16 @@ class PoserMeshedObject(StructuredAttribut):
 
     return res
 
-  # 
+  #
   # Return the alternate geom channel. Null if none
-  #    
+  #
   def getAltGeomChannel(self):
     return next( (gt for gt in self.getChannels() if gt.getPoserType()==PoserToken.E_geomChan), None)
 
 
   def getChannel(self, channelName):
     return self._channels.get(channelName.lower())
-  
+
   # Find the GenericTransform that represents a TargetGeom and that has the
   # given "channelName"
   # @param channelName
@@ -239,7 +246,7 @@ class PoserMeshedObject(StructuredAttribut):
     gt = self._channels.get(channelName.lower())
     return gt if gt and (gt.getPoserType()==PoserToken.E_targetGeom) else None
 
-  # 
+  #
   # Find the GenericTransform that represents the given "channelName" or PoserToken
   # @param channelName
   #          Channel Name to look for.
@@ -247,9 +254,9 @@ class PoserMeshedObject(StructuredAttribut):
   def getGenericTransform(self, channel:'Name Or PoserToken'):
     return next( (gt for gt in self.getChannels() if gt.getPoserType()==channel), None) if isinstance(channel, PToken) \
                   else self._channels.get(channel.lower())
-    
 
-  # 
+
+  #
   # Update or create a targetGeom and create a dependency to the body homonyme valueParm (if bodyLinked=true)
   #
   # @param channelName
@@ -262,7 +269,7 @@ class PoserMeshedObject(StructuredAttribut):
   #    *          List of deltas
   # @param bodyIdx
   #    *          Body index of the referenced figure (silly)
-  #    
+  #
   #     updateOrCreate(String channelName, String targetFigureName, WaveGeom3d srcWG, lstNewDeltas, bodyIdx, bodyLinked)
   # def updateOrCreate_0(self, channelName, targetFigureName, srcWG, lstNewDeltas, bodyIdx):
   def updateOrCreate(self, channelName, targetFigureName, srcWG, setNewDeltas, bodyIdx, bodyLinked=True):
@@ -288,7 +295,7 @@ class PoserMeshedObject(StructuredAttribut):
     gt.setNumbDeltas(len(srcWG.coordList))
     return gt
 
-  # 
+  #
   # Update or create a valueParm and initializes it
   # @param channelName ValueParm name
   def updateOrCreateVP(self, channelName, minVal=0.0, maxVal=1.0, applyLimits=True, isHidden=False):
@@ -309,58 +316,58 @@ class PoserMeshedObject(StructuredAttribut):
     vp.setHidden(isHidden)
     return vp
 
-  
+
 
   def createDefaultChannels(self, fp:'FigurePart'):
     ''' Create the 16 minimal channels for this actor and initialize them according to the 4 textual parameters.
     Textual parameters are supposed to be 'stripped'
-    
+
     Parameters
     ----------
     fp: FigurePart
       Description of the actor.
       Used fields are: geomGroup, translation, rotation, orientation, center
-    
+
     Return
     ------
     int : Return Code
-    
+
     '''
     ret = C_OK
-    
+
     try:
       AxisOrder, vOrient = fp.getOrientation()
-      
+
       # Get a copy of the default channels (we will modify the content)
       lstChan = PoserToken.getMinimalChannels(AxisOrder, not fp.geomGroup)
-      
+
       # Compute Actor's center
       bc = fp.getBaryCentre()
-        
+
       # Set the Origin and Copy it in the static channels xOffset...
       self.setOrigin(bc)
       PoserToken.setInitVect(lstChan, PoserToken.OFFSETA, bc)
       PoserToken.setInitVect(lstChan, PoserToken.OFFSETB, Point3d(bc).neg())
-  
+
       # Set Actor's orientation
       self.setOrientation(vOrient)
-      
+
       # Manage Translation
       tr = fp.getTranslation(self.getFigure(), bc, actor=self)
       PoserToken.setInitVect(lstChan, PoserToken.TRANS, tr)
-        
+
       # Manage Rotation
       vri = fp.getRotation()
       PoserToken.setInitVect(lstChan, PoserToken.ROT, vri)
-        
+
       # Create the list of channels
-      for gtdef in lstChan:    
+      for gtdef in lstChan:
         vp = GenericTransform.createFrom(gtdef)
         self.addAttribut(vp)
 
     except UsageError:
       #exc_type, exc_value, exc_tb = sys.exc_info()
-      #traceback.print_exception(exc_type, exc_value, exc_tb)       
+      #traceback.print_exception(exc_type, exc_value, exc_tb)
       ret = fp.ret
 
     return ret
@@ -370,7 +377,7 @@ class PoserMeshedObject(StructuredAttribut):
     snp = self.updateOrCreateVP(channelName, minVal=0.0, maxVal=1.0, applyLimits=True, isHidden=isHidden)
     snp.setPoserType(PoserToken.E_shaderNodeParm)
     return snp
-  
+
   def CreateDeformerP(self, channelName, zoneName, deformerName, isHidden=False):
     snp = self.updateOrCreateVP(channelName, minVal=-100.0, maxVal=100.0, applyLimits=True, isHidden=isHidden)
     snp.setPoserType(PoserToken.E_deformerPropChan)
@@ -384,13 +391,13 @@ class PoserMeshedObject(StructuredAttribut):
     dp.setValue(deformerName)
     snp.addAttribut(dp)
     return snp
-  
+
 
   def getBaseMesh(self, poserRootDir):
     if self._geom == None:
       self._geom = self.getBaseGeomCustom(poserRootDir).getWaveGeom()
     return self._geom
- 
+
   def printBaseGeomCustom(self):
     ''' Return a printable string representing the geometry '''
     s = PoserConst.GEOMPRINT[self._geomType]
@@ -398,7 +405,7 @@ class PoserMeshedObject(StructuredAttribut):
       # geomHandlerGeom 13 hip
       s = PoserConst.GEOMPRINT[self._geomType] + '[' + self._FRF.getValue() + ', ' + self._geomGroupName + ']'
     elif self._geomType==PoserConst.GT_LOCAL_OBJFILE:
-      # objFileGeom 0 0 :Runtime:Geometries:Marforno:Elaya:Lamp.obj          
+      # objFileGeom 0 0 :Runtime:Geometries:Marforno:Elaya:Lamp.obj
       s = PoserConst.GEOMPRINT[self._geomType] + '[' + self._geomFileName + ']'
     else: # Alternate should not appear here
       s = PoserConst.GEOMPRINT[self._geomType]
@@ -409,7 +416,7 @@ class PoserMeshedObject(StructuredAttribut):
     ''' Initialize the internal GeomCustom for a GT_LOCAL_OBJFILE '''
     if self._geomType==PoserConst.GT_LOCAL_OBJFILE:
       self._geomCustom = GeomCustom(src=wg)
-  
+
   def getBaseGeomCustom(self, poserRootDir):
     ''' Return the Geometry definition of the object. Without applying any morph.
     Returns
@@ -426,7 +433,7 @@ class PoserMeshedObject(StructuredAttribut):
         self._geomCustom = gc.extractSortGeom(self._geomGroupName, self._geomType) if gc else None
 
       elif self._geomType==PoserConst.GT_LOCAL_OBJFILE:
-        # objFileGeom 0 0 :Runtime:Geometries:Marforno:Elaya:Lamp.obj          
+        # objFileGeom 0 0 :Runtime:Geometries:Marforno:Elaya:Lamp.obj
         self._geomCustom = GeomCustom(getRealPath(poserRootDir, self._geomFileName))
       else:
         logging.warning("Unknown GeomType")
@@ -574,12 +581,12 @@ class PoserMeshedObject(StructuredAttribut):
 
   def setOrientation(self, v):
     self._orientation = v
-    
+
   # Customer Data Management
   def setCustomData(self, key, val, withPose=0, withMat=0):
     if not self._customData:
       self._customData = CustomData()
-      
+
     self._customData.set(key, val, withPose, withMat)
 
   def getCustomData(self, key=None):
@@ -589,7 +596,7 @@ class PoserMeshedObject(StructuredAttribut):
     '''
     if not self._customData:
       return None
-      
+
     return self._customData.get(key) if key else self._customData._data
 
   def hasCustomData(self):
@@ -624,7 +631,7 @@ class PoserMeshedObject(StructuredAttribut):
 
     setTargetMorph : set or list of str
       Morphs' names to create
-      
+
     ropt : ReportOption
       Tuple of options for the report algorithm
 
@@ -636,12 +643,12 @@ class PoserMeshedObject(StructuredAttribut):
       C_FILE_NOTFOUND : Some WaveFront files were not found
     '''
     ret = C_OK
-    
+
     if not setTargetMorph:
       return C_ERROR
 
     lstCurGeom = [ self.getBaseGeomCustom(poserRootDir), ]
-    
+
     if isinstance(refObj, PoserMeshedObject):
       # Extract and Sort GeomCustom to get single numbered groups
       lstRefGeom = [ refObj.getBaseGeomCustom(poserRootDir), ]
@@ -656,7 +663,7 @@ class PoserMeshedObject(StructuredAttribut):
       srcWG = srcGC.getWaveGeom()
 
       tabMapping = calcMapping_KDTree(srcWG, lstRefGeom, ropt.translation, ropt.maxDist)
-      
+
       self.findReportDeltas(refObj, srcGC, srcWG, tabMapping, setTargetMorph, ropt)
 
     return ret
@@ -681,7 +688,7 @@ class PoserMeshedObject(StructuredAttribut):
 
         if checkBODYParm:
           self.getFigure().updateOrCreateValueParm(channelName)
-          
+
         logging.info("Deltas created for %s.%s : %d", srcGC.getName(), channelName, len(lstFinalDeltas))
       else:
         logging.warn("No Delta created for %s.%s", srcGC.getName(), channelName )
@@ -698,14 +705,14 @@ class PoserMeshedObject(StructuredAttribut):
   # @return    C_OK when result is correct. C_ERROR when parameters are not correct.
   def createAlternateDeltas(self, poserRootDir, setTargetMorph, ropt, otherRefObjs=None):
     ret = C_OK
-    
+
     if not setTargetMorph:
       return C_ERROR
 
     curPoserObject = self.getPoserFile()
     refMeshedObj = self
 
-    # Find the actor      
+    # Find the actor
     srcActor = curPoserObject.findActor(self.getName())
 
     # Extract and load alternate geoms
@@ -716,7 +723,7 @@ class PoserMeshedObject(StructuredAttribut):
 
       # Find the geomChan Channel (should always exists if the list of alt geom is not empty)
       geomChan = self.getAltGeomChannel() # channelsAttr.findAttribut(PoserToken.E_geomChan)
-      
+
       altno = 1
       for altg in lstAltG:
         rfn = altg.getGeomFileName()
@@ -724,7 +731,7 @@ class PoserMeshedObject(StructuredAttribut):
         lstCurGeom.append(gc)
 
       lstRefGeom = [ refMeshedObj.getBaseGeomCustom(poserRootDir), ]
-      if otherRefObjs:        
+      if otherRefObjs:
         lstRefGeom += [ ro.getBaseGeomCustom(poserRootDir) for ro in otherRefObjs ]
 
       # Check File error
@@ -743,13 +750,13 @@ class PoserMeshedObject(StructuredAttribut):
           altChannelName = f'{channelName}_Alt{altno}'
           logging.info("Creating deltas for " + srcGC.getName() + "." + channelName + " into:" + altChannelName)
           setFoundDeltas = findNewDelta(tabMapping, refMeshedObj, channelName)
-          
+
           # Search accros other Reference actors
           if otherRefObjs:
             for ro in otherRefObjs:
               sfd = findNewDelta(tabMapping, ro, channelName)
               setFoundDeltas.update(sfd)
-            
+
           setNewDeltas = { }
 
           # Local Enhancement
@@ -760,12 +767,12 @@ class PoserMeshedObject(StructuredAttribut):
 
             # Create or update the channel list
             destGT = self.updateOrCreate(altChannelName, self.getName(), srcWG, setFinalDeltas, srcActor.getFigure().getBodyIndex(), False)
-            
+
             vod = ValueOpDelta(PoserToken.E_valueOpKey, srcActor.getFigure().getRefName(), srcActor.getName(), geomChan.getName(), 0.0)
 
             for ia in range(1, len(lstAltG)+1):
               vod.addValueKey(ia, 1.0 if (ia == altno) else 0.0)
-                       
+
             destGT.addVOD(vod)
 
             # Multiply by the value of the copied channel
@@ -783,11 +790,11 @@ class PoserMeshedObject(StructuredAttribut):
     '''Create a morph for the current PoserMeshedObject, based on an alternate geom.
     The morph applies to the base geometry or to an alternate one.
     Geometries shall have the same number of vertex (indexes matters)
-    
+
     Parameters
     ----------
       poserRootDir : str
-        Path to the poser data installation. (Usually c:\...\Poser9) 
+        Path to the poser data installation. (Usually c:\...\Poser9)
       morphGeomGroup : GeomGroup
         GeomGroup containing the morphed (translated) vertex
       targetMorphName : str
@@ -810,8 +817,8 @@ class PoserMeshedObject(StructuredAttribut):
     '''
     ret = C_OK
     curPoserObject = self.getPoserFile()
-    
-    # Find the actor      
+
+    # Find the actor
     srcActor = curPoserObject.findActor(self.getName())
 
     # Extract and load alternate geoms
@@ -819,17 +826,17 @@ class PoserMeshedObject(StructuredAttribut):
       lstAltG = self.getAltGeomList()
       if altGeomNo>len(lstAltG):
         return C_FAIL
-      
+
       # Find the geomChan Channel (should always exists if the list of alt geom is not empty)
       geomChan = self.getAltGeomChannel() # channelsAttr.findAttribut(PoserToken.E_geomChan)
-      
+
       rfn = lstAltG[altGeomNo-1].getGeomFileName()
       realpath = getRealPath(poserRootDir, rfn)
       refGeom = GeomCustom(realpath)
       if not refGeom or not refGeom.isValid():
         logging.warning('Alternate Geometry file not found:%s for %s', realpath, self.getName())
         return C_FILE_NOT_FOUND
-      
+
       refWG = refGeom.getWaveGeom()
       refGeomGrp = refWG.getGroups()[0]
     else: # Use base geom
@@ -842,7 +849,7 @@ class PoserMeshedObject(StructuredAttribut):
 
       # Need an extract and sort to get the rigth indexes
       refGeomGrp = refWG.extractSortGeom(refWG.groups[0].getName())
-      
+
     if len(refGeomGrp.coordList)!=len(morphGeomGroup.coordList):
       return C_BAD_DELTAINDEX
 
@@ -858,40 +865,40 @@ class PoserMeshedObject(StructuredAttribut):
         setNewDeltas[dp.noPt] = dp
 
     # Create or Update the TargetGeom
-    
+
     # Link to selected alternate geom if any
-    if geomChan:      
+    if geomChan:
       destGT = self.updateOrCreateVP(targetMorphName, minVal=0.0, maxVal=1.0, applyLimits=False)
 
       targetGT = self.updateOrCreate(f'{targetMorphName}_Alt{altGeomNo}', self.getName(), refGeomGrp, setNewDeltas, srcActor.getFigure().getBodyIndex(), False)
-      
+
       vod = ValueOpDelta(PoserToken.E_valueOpKey, srcActor.getFigure().getRefName(), srcActor.getName(), geomChan.getName(), 0.0)
 
       for ia in range(1, len(lstAltG)+1):
         vod.addValueKey(ia, 1.0 if (ia == altGeomNo) else 0.0)
-                       
+
       targetGT.addVOD(vod)
-      
+
       # Multiply by the value of the master channel
       vodMul = ValueOpDelta(PoserToken.E_valueOpTimes, srcActor.getFigure().getRefName(), srcActor.getName(), targetMorphName, 0.0)
       targetGT.addVOD(vodMul)
 
-      targetGT.setHidden(True)  
+      targetGT.setHidden(True)
     else:
-      destGT = self.updateOrCreate(targetMorphName, self.getName(), refGeomGrp, setNewDeltas, srcActor.getFigure().getBodyIndex(), False)    
-    
+      destGT = self.updateOrCreate(targetMorphName, self.getName(), refGeomGrp, setNewDeltas, srcActor.getFigure().getBodyIndex(), False)
+
     # If a masterMorphName is given: Create it at BODY level then link the destGT to it
     if masterMorphName:
       bodyAct = self.getFigure().getRootActor()
       bodyAct.updateOrCreateVP(masterMorphName, minVal=0.0, maxVal=1.0, applyLimits=False)
       vod = ValueOpDelta(PoserToken.E_valueOpDeltaAdd, srcActor.getFigure().getRefName(), bodyAct.getName(), masterMorphName, 1.0)
       destGT.addVOD(vod)
-    
+
     logging.info("Morph created for %s.%s : %d", refGeomGrp.getName(), targetMorphName, len(setNewDeltas))
 
     return ret
-    
-  # 
+
+  #
   # Return the delta for a given channel and a given vertex no.
   # @param channelName
   #          Channel Name (targetGeom name in fact)
@@ -902,7 +909,7 @@ class PoserMeshedObject(StructuredAttribut):
   def findDelta(self, channelName, refNo):
     gt = self.getTargetGeom(channelName)
     return gt.getDeltas().deltaSet.get(refNo) if gt and gt.getDeltas() else None
-          
+
 
   def deleteGTReference(self, grp, gt):
     #  FIX 2019-05-05 : Some (old) character may have not group
@@ -913,18 +920,18 @@ class PoserMeshedObject(StructuredAttribut):
         if po.getPoserType()==PoserToken.E_parmNode:
           if gt.getName()==po.getValue():
             grp.getLstAttr().remove(po)
-            
+
         elif po.getPoserType()==PoserToken.E_groupNode:
           #  Recursive, because a group may contain another group
           self.deleteGTReference(po, gt)
 
   def deleteChannel(self, gt):
     '''Delete the given channel.
-    
+
     Parameter:
     ----------
     gt : str or PoserGenericTransform
-      The channel to delete.    
+      The channel to delete.
     Returns
     -------
     int : C_OK when work done, C_FAIL if string input not found
@@ -935,15 +942,15 @@ class PoserMeshedObject(StructuredAttribut):
       gt = self._channels.get(gt.lower())
       if not gt:
         return C_FAIL
-      
+
     del self._channels[gt.getName().lower()]
     self.deleteGTReference(self._groups, gt)
     return C_OK
 
-  # 
+  #
   # Compute the first level list of channels that point to the given channel argument.
   # @param channel Reference channel
-  # @param setIn  Set to be extended and returned. Created if null. 
+  # @param setIn  Set to be extended and returned. Created if null.
   #               Does not contain duplicate channel.
   def getChannelDescendant(self, channel, setIn):
     implst = setIn if setIn else set()
@@ -956,7 +963,7 @@ class PoserMeshedObject(StructuredAttribut):
           break
 
     return implst
-  
+
   def getWeldParent(self):
     return self.getFigure().getWeldParent(self.getName())
 
@@ -985,7 +992,7 @@ class PoserMeshedObject(StructuredAttribut):
       res = C_NODELTA
     return res
 
-  # Change the name of referenced part (actor, prop, hairProp, controlProp)  
+  # Change the name of referenced part (actor, prop, hairProp, controlProp)
   # @param oldPartName
   # @param newPartName
   def changeReference(self, oldPartName, newPartName, changeRefToBODY=True):
@@ -1065,10 +1072,22 @@ class PoserMeshedObject(StructuredAttribut):
     elif tokenID==PoserToken.E_displayMode:
       self._displayMode = val
 
+    elif tokenID==PoserToken.E_subdivLevels:
+      self._subdivLevels = int(val)
+    elif tokenID==PoserToken.E_subdivRenderLevels:
+      self._subdivRenderLevels = int(val)
+    elif tokenID==PoserToken.E_backfaceBehavior:
+      self._backfaceBehavior = int(val)
+    elif tokenID==PoserToken.E_animatableOrientation:
+      self._animatableOrientation = (val[0] == "1")
+    elif tokenID==PoserToken.E_subdivideWithFigure:
+      self._subdivideWithFigure = (val[0] == "1")
+
+
     else:
-      logging.warning("Unexpected Direct: (%s) %s", tokenID.token, val)
+      logging.warning("Unexpected Direct: (%s) %s in (%s)", tokenID.token, val, self.getName())
+
 
-  
   # Overload add Attribut
   def addAttribut(self, po):
     if isinstance(po, GenericTransform):
@@ -1080,7 +1099,7 @@ class PoserMeshedObject(StructuredAttribut):
 
       self._channels[po.getName().lower()] = po
       po.setPoserMeshedObject(self)
-      
+
     elif po.getPoserType()==PoserToken.E_geomCustom:
       self._geomCustom = po
       self._geomType = PoserConst.GT_INTERNAL
@@ -1098,11 +1117,11 @@ class PoserMeshedObject(StructuredAttribut):
 
     elif po.getPoserType()==PoserToken.E_groups:
       self._groups = po
-      
+
     elif po.getPoserType()==PoserToken.E_readScript:
       #  TODO : Read Channel Definition Script
       logging.error("No Management rule for %s [%s]", po.getName(), po.getPoserType().token)
-      
+
     elif po.getPoserType()==PoserToken.E_customData:
       self._customData = po
 
@@ -1128,8 +1147,7 @@ class PoserMeshedObject(StructuredAttribut):
                 inchannels = True
                 continue
               else: # Syntax error
-                logging.warning("Line[%s] - read %s instead of '{'", st.lineno(), lw)
-                raise ParsingErrorException()
+                raise ParsingErrorException(f"L[{st.lineno()}]: read {lw} instead of '{{'")
 
             sta = create(vc, rw)
             self.addAttribut(sta)
@@ -1137,20 +1155,17 @@ class PoserMeshedObject(StructuredAttribut):
             code,cn,rw = st.getLine()
             if code==PoserFileParser.TT_LEFTBACKET:
               sta.read(st)
+            elif sta.isAmbi():
+              st.pushBack()
             else:
-              if sta.isAmbi():
-                st.pushBack()
-              else:
-                logging.warning("Line[%s]:  '{' is missing for %s", st.lineno(), cn)
-                raise ParsingErrorException()
+              raise ParsingErrorException(f"L[{st.lineno()}]: '{{' is missing for {cn}")
+          elif vc.isDirect:
+            self.setDirect(vc, rw)
           else:
-            if vc.isDirect:
-              self.setDirect(vc, rw)
-            else:
-              sa = create(vc, cn)
-              #  Read before add
-              sa.read(st, rw)
-              self.addAttribut(sa)
+            sa = create(vc, cn)
+            #  Read before add
+            sa.read(st, rw)
+            self.addAttribut(sa)
         except KeyError:  #  Mot inconnu
           logging.info("Line[%s] - Unknown word:%s", st.lineno(), cn)
 
@@ -1162,8 +1177,7 @@ class PoserMeshedObject(StructuredAttribut):
       elif code==PoserFileParser.TT_EOF:
         break
       else:
-        logging.warning("Line[%s] - Not Accepted :%s", st.lineno(), cn)
-        raise ParsingErrorException()
+        raise ParsingErrorException(f'L[{st.lineno()}]: Not Accepted :{cn}')
 
     # End of read
 
@@ -1192,13 +1206,17 @@ class PoserMeshedObject(StructuredAttribut):
     fw.write(f"{nextPfx}name {self._pmName}\n")
     fw.write(f"{nextPfx}{'on' if self._visible else 'off'}\n")
     WBoolLine(fw, nextPfx, "bend", self._bend)
-    
-    if self.getPoserFile().isVersion(PoserConst.POSER_V9):
+
+    if self.getPoserFile().isBetterVersion(PoserConst.POSER_V9):
       WBoolLine(fw, nextPfx, "animatableOrigin", self._animatableOrigin)
 
+    if self.getPoserFile().isBetterVersion(PoserConst.POSER_V11):
+      WBoolLine(fw, nextPfx, "animatableOrientation", self._animatableOrientation)
+
+
     WBoolLine(fw, nextPfx, "dynamicsLock", self._dynamicsLock)
 
-    if self.getPoserFile().isVersion(PoserConst.POSER_V9):
+    if self.getPoserFile().isBetterVersion(PoserConst.POSER_V9):
       WBoolLine(fw, nextPfx, "collisionDetection", self._collisionDetection )
 
     WBoolLine(fw, nextPfx, "hidden", self._hidden )
@@ -1217,10 +1235,16 @@ class PoserMeshedObject(StructuredAttribut):
 
     fw.write(f"{nextPfx}creaseAngle {self._creaseAngle}\n")
     
+    if self.getPoserFile().isBetterVersion(PoserConst.POSER_V11):
+      fw.write(f"{nextPfx}subdivLevels {self._subdivLevels}\n")
+      fw.write(f"{nextPfx}subdivRenderLevels {self._subdivRenderLevels}\n")
+      WBoolLine(fw, nextPfx, "subdivideWithFigure", self._subdivideWithFigure )
+      fw.write(f"{nextPfx}backfaceBehavior {self._backfaceBehavior}\n")
+
     # Print WeightMap before channels
     for po in self._lstWMap:
       po.write(fw, nextPfx)
-    
+
     # Channels { } part
     chanPfx = nextPfx + "\t"
     fw.write(f"{nextPfx}channels\n{chanPfx}{{\n")
@@ -1243,13 +1267,13 @@ class PoserMeshedObject(StructuredAttribut):
         fw.write(f"{nextPfx}displayMode {self._displayMode}\n")
     fw.write(f"{nextPfx}customMaterial {self._customMaterial}\n")
     WBoolLine(fw, nextPfx, "locked", self._locked )
-    
+
     # Print other found attributes
     for po in self._lstAttr:
       po.write(fw, nextPfx)
     if not self._lstAttr:
       fw.write('\n')
-      
+
     # Write Customer Data
     if self._customData:
       self._customData.write(fw, nextPfx)
@@ -1266,16 +1290,16 @@ class PoserMeshedObject(StructuredAttribut):
 # Descendant classes of PoserMeshedObject
 # -----------------------------------------------------------------------------
 
-# 
+#
 # Just to avoid interactions with actors or props that have geoms
-#  
+#
 class Camera(PoserMeshedObject):
   def __init__(self):
     super(Camera, self).__init__()
 
-# 
+#
 # Just to avoid interactions with actors or props that have geoms
-#  
+#
 class Light(PoserMeshedObject):
   def __init__(self):
     super(Light, self).__init__()
@@ -1283,20 +1307,20 @@ class Light(PoserMeshedObject):
   def getLightType(self):
     sa = self.findAttribut(PoserToken.E_lightType)
     return sa.getIntegerValue()
-  
+
   def addAttribut(self, po):
     if po.getPoserType()==PoserToken.E_depthMapSize:
       prev = self.getGenericTransform(PoserToken.E_depthMapSize)
       if prev:
         # The second defintion should be the Simple attribut
         self.getLstAttr().append(po)
-        return         
-      
+        return
+
     PoserMeshedObject.addAttribut(self, po)
 
-# 
+#
 #  Describes a Poser "actor" object.
-#  
+#
 class PoserActor(PoserMeshedObject):
 
   def __init__(self, Name = None):
@@ -1327,9 +1351,9 @@ class PoserActor(PoserMeshedObject):
     self._castsShadow = False
     self._includeInDepthCue = False
 
-# 
+#
 # Represents a Poser "prop" object
-#  
+#
 class PoserProp(PoserMeshedObject):
   def __init__(self):
     super(PoserProp, self).__init__()
@@ -1340,7 +1364,7 @@ class PoserProp(PoserMeshedObject):
     self.idx = index(n)
 
   def getIndex(self):
-    ''' Return the figure index or 0 if the prop is outside any figure ''' 
+    ''' Return the figure index or 0 if the prop is outside any figure '''
     return self.idx
 
   def setIndex(self, ni):
@@ -1378,14 +1402,14 @@ class BaseProp(PoserProp):
   def setIndex(self, ni):
     ''' Set a new figure index for the BaseProp. '''
     super(BaseProp, self).setIndex(ni)
-    
+
     dp = self.getAttribut(PoserToken.E_deformerProp)
     dp.setValue( chgIndex(dp.getValue(), ni))
 
-# 
+#
 # Represents the Magnet object of a Magner deformer.
 # Just to avoid interactions with actors or props that have geoms
-#  
+#
 class MagnetDeformerProp(PoserProp):
   def __init__(self):
     super(MagnetDeformerProp, self).__init__()
@@ -1393,7 +1417,7 @@ class MagnetDeformerProp(PoserProp):
   def setIndex(self, ni):
     ''' Set a new figure index for the BaseProp. '''
     super(MagnetDeformerProp, self).setIndex(ni)
-    
+
     for po in self._lstAttr:
       if po.getPoserType() in (PoserToken.E_deformTarget, PoserToken.E_addZone ):
         po.setValue( chgIndex(po.getValue(), ni))
@@ -1406,26 +1430,26 @@ class SphereZoneProp(PoserProp):
   def __init__(self):
     super(SphereZoneProp, self).__init__()
 
-# 
+#
 # Represents a Poser "path" object
-#  
+#
 class CurveProp(PoserProp):
   def __init__(self):
     super(CurveProp, self).__init__()
 
-# 
-# 
+#
+#
 # Represents a Poser "Wave" object
-#  
+#
 class WaveDeformerProp(PoserProp):
   def __init__(self):
     super(WaveDeformerProp, self).__init__()
 
 
-# 
-# 
+#
+#
 # Represents a Poser "Wind Field" object
-#  
+#
 class ConeForceFieldProp(PoserProp):
   def __init__(self):
     super(ConeForceFieldProp, self).__init__()
@@ -1433,28 +1457,42 @@ class ConeForceFieldProp(PoserProp):
 
 
 # Just to avoid interactions with actors or props that have geoms
-#  
+#
 class ControlProp(PoserProp):
   def __init__(self):
     super(ControlProp, self).__init__()
 
+#
+class ControlHandleProp(PoserActor):
+  ''' Class discovered in Poser11 character.
+  Should be considered as Actor
+  '''
+  def __init__(self):
+    super(ControlHandleProp, self).__init__()
+    self._actorToControl = None
 
+  def setDirect(self, tokenID, val):
+    if tokenID==PoserToken.E_actorToControl:
+      self._actorToControl = val
+    else:
+      super(ControlHandleProp, self).setDirect(tokenID, val)
 
-# 
+  def write(self, fw, pfx):
+    if self._actorToControl:
+      fw.write(f"{pfx}{self.getPoserType().token} {self.getValue()}\n")
+    super(ControlHandleProp, self).write(fw, pfx)
+
+#
 # This class represents the hairProp attribut of a Poser File.
-#  
+#
 class HairProp(PoserProp):
   def __init__(self):
     super(HairProp, self).__init__()
     self._isSimple = True
     self.ambiguious = True
 
-    def __init__(self):
-        super(HairProp, self).__init__()
-
-  #    * @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
-    if self._isSimple:    
+    if self._isSimple:
       fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
 
   def addAttribut(self, po):
