@@ -150,7 +150,7 @@ class SphereMathsRaw(SimpleAttribut):
     for i in range(0, 2):
       self.m[i] = ''
       for j in range(0,4):
-        code, val = st.getFullLine()
+        _, val = st.getFullLine()
         self.m[i] += val + '\n'
        
     self._value = "\n\t" + self.m[0] + "\n" + self.m[1]
@@ -159,29 +159,29 @@ class SphereMathsRaw(SimpleAttribut):
 # Tuple Key/Value for OpKey operation
 valueKey = namedtuple('valueKey', ['key', 'val'])
 
-# 
-#  * This class represents a Poser Mathematical operation between channels
-#  * 
-#  
+# Updated for Poser11 support
 class ValueOpDelta(SimpleAttribut):
-  #  Should not be used, because SimpleAttribut stores already the tokenID
-  #  PTK deltaType = null;
-  # 
-  #    *  Default constructor needed by StructuredAttribut.
+  ''' This class represents a Poser Mathematical operation between channels '''
 
-  # if figure and actor not null
-  #   * Create an operation from a text expression
-  #   * +QualifiedChannelName
-  #   * -QualifiedChannelName
-  #   * +VAL*QualifiedChannelName
-  #   * 
-  #   * QualifiedChannelName = [ActorName [':' bodyIndex] '.' ]ChannelName
-  #   * 
-  # public ValueOpDelta(PoserToken deltaType, String figureName, String actorName, String channelName, float ctrlRatio)
-  # public ValueOpDelta(Figure fig, PoserActor curAct, String expr)
   def __init__(self, deltatype=None, pfigure=None, pactor=None, channelExpr=None, ctrlRatio=None, keys:'list of (key,val)'=None, src=None):
+    ''' Constructor. 
+    if figure and actor not null
+     * Create an operation from a text expression
+     * +QualifiedChannelName
+     * -QualifiedChannelName
+     * +VAL*QualifiedChannelName
+     * 
+     * QualifiedChannelName = [ActorName [':' bodyIndex] '.' ]ChannelName
+     * 
+    public ValueOpDelta(PoserToken deltaType, String figureName, String actorName, String channelName, float ctrlRatio)
+    public ValueOpDelta(Figure fig, PoserActor curAct, String expr)
+    '''
     super(ValueOpDelta, self).__init__(deltatype if deltatype else src.getPoserType() if src else None)
     self.lstValKeys = [ ]
+    
+    # V11 attribut for any kind of Ops
+    # Just record the given float
+    self._strength = None 
 
     if pfigure and isinstance(pfigure, str) and pactor and channelExpr:
       self.l0 = pfigure
@@ -239,7 +239,7 @@ class ValueOpDelta(SimpleAttribut):
   
       ptind = qualifiedName.find('.')
       if ptind < 0:
-        self.l0 = "Figure " + str(pfigure.getBodyIndex())
+        self.l0 = f"Figure {pfigure.getBodyIndex()}"
         self.l1 = pactor.getName()
         self.l2 = qualifiedName
       else:
@@ -247,7 +247,7 @@ class ValueOpDelta(SimpleAttribut):
         
         ptdp = qualifiedName.find(':')
         if (ptdp < 0):
-          self.l1 += ':' + str(pfigure.getBodyIndex())
+          self.l1 += f':{pfigure.getBodyIndex()}'
   
         self.l0 = "Figure " + str(index(self.l1))
         self.l2 = qualifiedName[ptind + 1:]
@@ -269,7 +269,7 @@ class ValueOpDelta(SimpleAttribut):
       self.l2 = ''
       self.controlRatio = 0.0
 
-    self._value = self.l0 + "\n" + self.l1 + "\n" + self.l2
+    self._value = f"{self.l0}\n{self.l1}\n{self.l2}"
 
 
   def calc(self, curv, parentv):
@@ -329,56 +329,63 @@ class ValueOpDelta(SimpleAttribut):
     self.lstValKeys.append(vk)
 
   def read(self, st, val):
-    code, self.l0 = st.getFullLine()
-    code, self.l1 = st.getFullLine()
+    _, self.l0 = st.getFullLine()
+    _, self.l1 = st.getFullLine()
     code, self.l2 = st.getFullLine()
     
+    if st.getFileVersion()>=PoserConst.POSER_V11f:
+      # Consider reading the line strength
+      # As of 2022-04-07 : Usage / Management unknown
+      code,cn,rw = st.getLine()
+      if code==PoserFileParser.TT_WORD and Lang.get(cn,None)==PoserToken.E_strength: 
+        self._strength = float(rw)
+      else:
+        logging.warning(f"L[{st.lineno()}] - Missing strength in {st.sval}")
+        st.pushBack()
+    
     if self.getPoserType()==PoserToken.E_valueOpDeltaAdd:
       code,cn,rw = st.getLine() #  Should get 'deltaAddDelta'      
       self.controlRatio = float(rw)
-    else:
-      if self.getPoserType() == PoserToken.E_valueOpKey:
-        code,cn,rw = st.getLine() #  Should get 'beginValueKeys'
-        #  While next token != endValueKeys : Store the keys        
-        while True:
-          code,cn,rw = st.getLine()
-          if code==PoserFileParser.TT_WORD:
-            try:
-              vc = Lang[cn]
+    elif self.getPoserType() == PoserToken.E_valueOpKey:
+      code,cn,rw = st.getLine() #  Should get 'beginValueKeys'
+      #  While next token != endValueKeys : Store the keys        
+      while True:
+        code,cn,rw = st.getLine()
+        if code==PoserFileParser.TT_WORD:
+          vc = Lang.get(cn, None)
 
-              if vc==PoserToken.E_valueKey:  
-                tv = rw.split()                    
-                self.addValueKey(float(tv[0]), float(tv[1]))
-              elif vc==PoserToken.E_endValueKeys:
-                break
-              else:
-                logging.warning("L[" + st.lineno() + "] - Not Accepted :" + st.sval)
-                continue 
-            except KeyError:
-              logging.warning("L[" + st.lineno() + "] - Not Accepted :" + st.sval)
-              raise ParsingErrorException()
-          elif code==PoserFileParser.TT_EOF: 
+          if vc==PoserToken.E_valueKey:  
+            tv = rw.split()                    
+            self.addValueKey(float(tv[0]), float(tv[1]))
+          elif vc==PoserToken.E_endValueKeys:
             break
+          else:
+            raise ParsingErrorException(f"L[{st.lineno()}]: Not Accepted:{st.sval}")
+          
+        elif code==PoserFileParser.TT_EOF: 
+          break
+    #else: Nothing to read for other operation
+    
+    _value = f"{self.l0}\n{self.l1}\n{self.l2}"
 
-    _value = self.l0 + "\n" + self.l1 + "\n" + self.l2
-
-  #    * @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
-  #    
   def write(self, fw, pfx):
-    fw.write(pfx + self.getName() + '\n')
+    fw.write(f'{pfx}{self.getName()}\n')
     #  FIX 20140329 - Use indentation to avoid some cross bug in Poser7
-    #  fw.write(pfx + "\t" + getValue());
-    fw.write(pfx + "\t" + self.l0 + '\n')
-    fw.write(pfx + "\t" + self.l1 + '\n')
-    fw.write(pfx + "\t" + self.l2 + '\n')
-    #  fw.write(pfx + "\t" + getValue().replaceAll("\n", pfx+"\n"));
+    
+    # Optim
+    pfx += "\t"
+    fw.write(f'{pfx}{self.l0}\n{pfx}{self.l1}\n{pfx}{self.l2}\n')
+    
+    if self._strength is not None:
+      fw.write(f'{pfx}strength {self._strength}\n')
+    
     if self.getPoserType()==PoserToken.E_valueOpDeltaAdd:
-      fw.write(pfx + "\tdeltaAddDelta " + str(self.controlRatio) + '\n')
+      fw.write(f'{pfx}deltaAddDelta {self.controlRatio}\n')
     elif self.getPoserType()==PoserToken.E_valueOpKey:
-      fw.write(pfx + '\tbeginValueKeys\n')
+      fw.write(f'{pfx}beginValueKeys\n')
       for vk in self.lstValKeys:
-        fw.write(pfx + "\t  valueKey " + str(vk.key) + " " + str(vk.val) + '\n')
-      fw.write(pfx + '\tendValueKeys\n')
+        fw.write(f'{pfx}  valueKey {vk.key} {vk.val}\n')
+      fw.write(f'{pfx}endValueKeys\n')
 
   def getFigureName(self): return self.l0
 
@@ -386,7 +393,7 @@ class ValueOpDelta(SimpleAttribut):
 
   def setGroupName(self, gn):
     self.l1 = gn
-    self._value = self.l0 + "\n" + self.l1 + "\n" + self.l2
+    self._value = f'{self.l0}\n{self.l1}\n{self.l2}'
 
   def getChannelName(self): return self.l2
 
@@ -471,9 +478,9 @@ class LinkParmsSA(SimpleAttribut):
     
   def read(self, st, val):
     self.MasterGroupName = val
-    code,self.MasterChannelName = st.getFullLine()
-    code,self.SlaveGroupName = st.getFullLine()    
-    code,self.SlaveChannelName = st.getFullLine()
+    _,self.MasterChannelName = st.getFullLine()
+    _,self.SlaveGroupName = st.getFullLine()    
+    _,self.SlaveChannelName = st.getFullLine()
     self._value = self.MasterGroupName + "\n\t\t" + self.MasterChannelName + "\n\t\t" + self.SlaveGroupName + "\n\t\t" + self.SlaveChannelName
 
   def getChildName(self): return self.getName()
@@ -481,14 +488,5 @@ class LinkParmsSA(SimpleAttribut):
   def getParentName(self): return self.MasterChannelName
 
 
-#  
-# LinkWeight attribut.
-#   class LinkWeightSA(SimpleAttribut):
-#  
-# Just read the entire line
-#  class NameSA(SimpleAttribut):
 
 
-# if __name__ == '__main__':
-#   print(dir(SimpleAttribut))
-
