@@ -153,7 +153,7 @@ class PoserFileParser(object):
       else:
         raise Exception("Parsing Error in line:"+str(self.LINENO)+" Not a Vertex")
 
-    except ParsingErrorException as pex:
+    except ValueError as pex:
       # Raise added on 20201013 (TBC)
       raise Exception("Parsing Error in line:"+str(self.LINENO)+" Not a Vertex")
 
@@ -170,7 +170,7 @@ class PoserFileParser(object):
       else:
         raise Exception("Parsing Error in line:"+str(self.LINENO)+" Not a Texture")
 
-    except ParsingErrorException as pex:
+    except ValueError as pex:
       raise Exception("Parsing Error in line:"+str(self.LINENO)+" Not a Texture")
 
     #self.skipToNextLine()
@@ -186,7 +186,7 @@ class PoserFileParser(object):
       else:
         raise Exception("Parsing Error in line:"+str(self.LINENO)+" Not a Vector3d")
 
-    except ParsingErrorException as pex:
+    except ValueError as pex:
       raise Exception("Parsing Error in line:"+str(self.LINENO)+" Not a Vector3d")
 
     #self.skipToNextLine()
