@@ -1307,16 +1307,6 @@ def CreateTransform(eu, ev):
 
   return ew, M, MT
 
-# This method refers GeomGroup methods and should be implemented elsewhere
-def CreatePlaneDef(planes, planeName, orientation='XZY'):
-  ''' Find the center of the plane and the two vectors that define the plane '''
-  plane = planes.getGroup(planeName)
-  vxtab = plane.getFaceVertex(0)
-  eu = Vector3d( vxtab[3].x - vxtab[0].x, vxtab[3].y - vxtab[0].y, vxtab[3].z - vxtab[0].z).normalize()
-  ev = Vector3d( vxtab[1].x - vxtab[0].x, vxtab[1].y - vxtab[0].y, vxtab[1].z - vxtab[0].z).normalize()       
-  center = Point3d(vxtab[3].x + vxtab[1].x, vxtab[3].y + vxtab[1].y, vxtab[3].z + vxtab[1].z).scale(0.5)
-  
-  return center, eu, ev
 
 
 
