@@ -283,7 +283,7 @@ class WaveGeom(object):
       elif isinstance(groups, GeomGroup):
         lstGrps.append(groups)
       else:
-        lstGrps = [ (g if isinstance(g, GeomGroup) else self.getGroup(g)) for g in lstGrps ]
+        lstGrps = [ (g if isinstance(g, GeomGroup) else self.getGroup(g)) for g in groups ]
       
     else:
       lstGrps = self.groups
@@ -975,13 +975,13 @@ class WaveGeom(object):
       fout.close()
 
     except FileNotFoundError:
-      if WFBasic.PYPOS3D_TRACE: print('File Not Found Error:' + fileName)
+      if WFBasic.PYPOS3D_TRACE: print(f'File Not Found Error:{fileName}')
       ret = C_ERROR
 
     except OSError as e: # (IOException ioex)
       if WFBasic.PYPOS3D_TRACE: 
         traceback.print_last()
-        print('Write Error:' + str(e))
+        print(f'Write Error:{e}')
       ret = C_ERROR
       
     finally:
@@ -1009,13 +1009,13 @@ class WaveGeom(object):
       fout.close()
 
     except FileNotFoundError:
-      if WFBasic.PYPOS3D_TRACE: print('File Not Found Error:' + fileName)
+      if WFBasic.PYPOS3D_TRACE: print(f'File Not Found Error:{fileName}')
       ret = C_ERROR
       
     except OSError as e: # (IOException ioex)
       if WFBasic.PYPOS3D_TRACE: 
         traceback.print_last()
-        print('Write Error:' + str(e))
+        print(f'Write Error:{e}')
       ret = C_ERROR
       
     finally:
@@ -1058,10 +1058,10 @@ class WaveGeom(object):
   # @return the table of index of the edge vertex
   #
   #  public int[] findEdgeCoord()
-  def findEdgeCoord(self):
-    gName = self.getGroupName()
-    ng = self.getGroup(gName)
-    return ng.findEdgeCoord()
+  # def findEdgeCoord(self):
+  #   gName = self.getGroupName()
+  #   ng = self.getGroup(gName)
+  #   return ng.findEdgeCoord()
 
 
   #  public int[] extractSortJonction(String pBasName, String pHautName)
@@ -1582,6 +1582,16 @@ class WaveGeom(object):
     return ng
 
 
+  # This method refers GeomGroup methods and should be implemented elsewhere
+  def CreatePlaneDef(self, planeName, orientation='XZY'):
+    ''' Find the center of the plane and the two vectors that define the plane '''
+    plane = self.getGroup(planeName)
+    vxtab = plane.getFaceVertex(0)
+    eu = Vector3d( vxtab[3].x - vxtab[0].x, vxtab[3].y - vxtab[0].y, vxtab[3].z - vxtab[0].z).normalize()
+    ev = Vector3d( vxtab[1].x - vxtab[0].x, vxtab[1].y - vxtab[0].y, vxtab[1].z - vxtab[0].z).normalize()       
+    center = Point3d(vxtab[3].x + vxtab[1].x, vxtab[3].y + vxtab[1].y, vxtab[3].z + vxtab[1].z).scale(0.5)
+  
+    return center, eu, ev
 
 
   def fillHole(self, srcGrpOrName, srcMatName, destGrpName, destMatName, mergeGrp, nbLoop, alpha, createCenter=True):
@@ -2008,9 +2018,9 @@ def PlaneCut(target, centerOrRepOrPlane, eu=None, ev=None, materialName='Section
     startidx = target.getFaceStartIdx(faceno)
     lastidx = target.getFaceLastIdx(faceno)
                         
-    if lastidx-startidx<3:
-      if WFBasic.PYPOS3D_TRACE: print(f'  PlaneCut[{target.getName()}] ignoring face: {faceno}')
-      continue
+    # if lastidx-startidx<3:
+    #   if WFBasic.PYPOS3D_TRACE: print(f'  PlaneCut[{target.getName()}] ignoring face: {faceno}')
+    #   continue
     
     # Retrieve the face attributs (Text,Norm, material) of the face to cut
     fattr = fao.fAttr
@@ -2203,9 +2213,9 @@ def PlaneSplit(target, centerOrRepOrPlane, eu=None, ev=None, radialLimit=0.0, ma
     startidx = target.getFaceStartIdx(faceno)
     lastidx = target.getFaceLastIdx(faceno)
                         
-    if lastidx-startidx<3:
-      if WFBasic.PYPOS3D_TRACE: print(f'  [{target.getName()}] ignoring face: {faceno}')
-      continue
+    # if lastidx-startidx<3:
+    #   if WFBasic.PYPOS3D_TRACE: print(f'  [{target.getName()}] ignoring face: {faceno}')
+    #   continue
     
     # Retrieve the material and the attributs of the face to cut
     fattr = fao.fAttr
@@ -2398,9 +2408,9 @@ def PlaneSlice(target, centerOrRepOrPlane, eu=None, ev=None, cutFaceMatLst=None,
     fattr = fao.fAttr
     hasTexture = (fattr & C_FACE_TEXT)!=0
     
-    if lastidx-startidx<3:
-      if WFBasic.PYPOS3D_TRACE: print(f'PlaneSlice[{target.getName()}] ignoring NON face: {faceno}')
-      continue
+    # if lastidx-startidx<3:
+    #   if WFBasic.PYPOS3D_TRACE: print(f'PlaneSlice[{target.getName()}] ignoring NON face: {faceno}')
+    #   continue
     
     # Compute the list of enhanced Vertex, without aligned ones
     # We have at least 3 points
