@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # To be executed before SVN commit
-tulist="tuPaveList.py tuPlaneCut.py tuGeomGroup.py tuWaveGeom.py tuPoserMeshed.py tuFigure.py tuPoserFile.py tuQSlim.py tuWFBasic.py"
+tulist="tuPaveList.py tuPlaneCut.py tuGeomGroup.py tuWaveGeom.py tuPoserMeshed.py tuFigure.py tuPoserFile.py tuQSlim.py tuWFBasic.py tuPoser11"
 
 tuPyVM="python3.9 python3.6"
 
